#include <msp/io/print.h>
#include <msp/strings/utils.h>
#include "statement.h"
#include "textwriter.h"

using namespace std;

namespace Msp {
namespace DataFile {

TextWriter::TextWriter(IO::Base &o):
	WriterMode(o),
	float_format("%#.7g")
{ }

void TextWriter::set_float_precision(unsigned fp)
{
	float_format = format("%%#.%dg", fp/4-1);
}

void TextWriter::write(const Statement &st)
{
	write_(st, 0);
}

void TextWriter::write_(const Statement &st, unsigned level)
{
	string indent(level, '\t');

	IO::print(out, "%s%s", indent, st.keyword);
	for(ValueArray::const_iterator i = st.args.begin(); i!=st.args.end(); ++i)
	{
		out.put(' ');
		if(i->get_signature()==StringType::signature)
			IO::print(out, "\"%s\"", c_escape(i->get<StringType::Store>(), false));
		else if(i->get_signature()==BoolType::signature)
			out.write(i->get<BoolType::Store>() ? "true" : "false");
		else if(i->get_signature()==IntType::signature)
			out.write(lexical_cast(i->get<IntType::Store>()));
		else if(i->get_signature()==FloatType::signature)
			out.write(format(float_format, i->get<FloatType::Store>()));
		else if(i->get_signature()==SymbolType::signature)
		{
			string name = i->get<SymbolType::Store>().name;
			if(isdigit(name[0]))
				out.write("\\"+name);
			else
				out.write(name);
		}
	}
	if(!st.sub.empty())
	{
		IO::print(out, "\n%s{\n", indent);
		for(list<Statement>::const_iterator i = st.sub.begin(); i!=st.sub.end(); ++i)
			write_(*i, level+1);
		IO::print(out, "%s}", indent);
	}
	out.write(";\n", 2);
}

} // namespace DataFile
} // namespace Msp
